package cz.fi.muni;

import cz.fi.muni.internalHarvesting.FieldInternal;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import cz.fi.muni.harvesting.api.FieldApiDelegate;
import cz.fi.muni.harvesting.model.*;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Component;

import java.time.LocalDate;
import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

@Component
public class FieldController implements FieldApiDelegate {
    private static final Logger log = LoggerFactory.getLogger(FieldController.class);

    @Override
    public ResponseEntity<NewFieldResponse> newField(NewField newField) {
        if (newField == null) {
            log.info("No New Field to be added provided.");
            return new ResponseEntity<>(null, HttpStatus.BAD_REQUEST);
        }
        // Generate id
        UUID newFieldUUID = UUID.randomUUID();
        // Add to DB
        boolean fieldAdded = true;
        // Handle bad Field
        if (!fieldAdded) {
            log.info("Could not create a new Field");
            return new ResponseEntity<>(null, HttpStatus.UNPROCESSABLE_ENTITY);
        }
        NewFieldResponse newFieldResponse = new NewFieldResponse() {
            {
                message(newFieldUUID.toString());
            }
        };
        return new ResponseEntity<>(newFieldResponse, HttpStatus.OK);
    }
    @Override
    public ResponseEntity<Field> getField(String id) {
        if (id == null) {
            log.info("Provided Field id is null.");
            return new ResponseEntity<>(null, HttpStatus.NOT_FOUND);
        }
        // retrieve field from DB
        Field field = new Field();
        field.setVariety("Pálava");
        field.setHarvestSize(200);
        field.setHarvestDate(LocalDate.parse("2023-08-08"));

        // handle non-existing id
        if (field == null) {
            log.info("Could not find a Field with this id .");
            return new ResponseEntity<>(null, HttpStatus.NOT_FOUND);
        }
        return new ResponseEntity<>(field, HttpStatus.OK);
    }
    @Override
    public ResponseEntity<Void> deleteField(String id) {
        if (id == null) {
            log.info("Provided Field id is null.");
            return new ResponseEntity<>(null, HttpStatus.NOT_FOUND);
        }
        // Remove from DB
        boolean fieldDeleted = true;
        // Handle deletion error
        if (!fieldDeleted) {
            log.info("Deletion of this Field was not successful.");
            return new ResponseEntity<>(null, HttpStatus.UNPROCESSABLE_ENTITY);
        }
        return new ResponseEntity<>(HttpStatus.OK);
    }

    @Override
    public ResponseEntity<Void> editField(String id, EditField editField) {
        if (id == null) {
            log.info("Provided Field id is null.");
            return new ResponseEntity<>(null, HttpStatus.NOT_FOUND);
        }
        // Edit in DB
        boolean fieldEdited = true;
        if (!fieldEdited) {
            log.info("Could not edit Field with this id ");
            return new ResponseEntity<>(null, HttpStatus.UNPROCESSABLE_ENTITY);
        }
        return new ResponseEntity<>(HttpStatus.OK);
    }

    @Override
    public ResponseEntity<HarvestedFieldResponse> harvestField(String id) {

        if (id == null) {
            log.info("No Field id provided.");
            return new ResponseEntity<>(null, HttpStatus.BAD_REQUEST);
        }
        // Harvest
        log.info("Harvesting field");
        // Check if id is valid
        if (!id.equals("dbd055c6-b7bb-47d7-b95a-37976f114e52")) {
            log.info("Using temporary default UUID value");
            id = "dbd055c6-b7bb-47d7-b95a-37976f114e52";
        }
        UUID newlyHarvestedRawGrapeId = FieldInternal.harvestField(UUID.fromString(id));
        log.info("harvested");

        // Handle bad Field
        if (newlyHarvestedRawGrapeId == null) {
            log.info("Failed to harvest Field");
            return new ResponseEntity<>(null, HttpStatus.UNPROCESSABLE_ENTITY);
        }
        HarvestedFieldResponse harvestedFieldResponse = new HarvestedFieldResponse() {
            {
                message(newlyHarvestedRawGrapeId.toString());
            }
        };
        return new ResponseEntity<>(harvestedFieldResponse, HttpStatus.OK);
    }

    @Override
    public ResponseEntity<List<Field>> getAllFields() {
        // Retrieve all Fields from DB
        log.info("getAllFields");
        List<Field> fields = new ArrayList<>();
        Field f1 = new Field();
        f1.setVariety("Pálava");
        f1.setHarvestSize(200);
        f1.setHarvestDate(LocalDate.parse("2023-08-08"));
        fields.add(f1);
        Field f2 = new Field();
        f2.setVariety("Chardonay");
        f2.setHarvestSize(1000);
        f2.setHarvestDate(LocalDate.parse("2023-09-09"));
        fields.add(f2);
        return new ResponseEntity<>(fields, HttpStatus.OK);
    }
}



