package cz.fi.muni;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import cz.fi.muni.harvesting.api.RawGrapeApiDelegate;
import cz.fi.muni.harvesting.model.*;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Component;


import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

@Component
public class RawGrapeController implements RawGrapeApiDelegate {
    private static final Logger log = LoggerFactory.getLogger(RawGrapeController.class);

    @Override
    public ResponseEntity<AddRawGrapeResponse> addRawGrape(NewRawGrape newRawGrape) {
        if (newRawGrape == null) {
            log.info("No New Raw Grape to be added provided.");
            return new ResponseEntity<>(null, HttpStatus.BAD_REQUEST);
        }
        // Generate id
        UUID newRawGrapeUUID = UUID.randomUUID();
        // Add to DB
        boolean rawGrapeAdded = true;
        // Handle bad grape
        if (!rawGrapeAdded) {
            log.info("Could not create a new Raw Grape");
            return new ResponseEntity<>(null, HttpStatus.UNPROCESSABLE_ENTITY);
        }
        AddRawGrapeResponse addRawGrapeResponse = new AddRawGrapeResponse() {
            {
                message(newRawGrapeUUID.toString());
            }
        };
        return new ResponseEntity<>(addRawGrapeResponse, HttpStatus.OK);
    }
    @Override
    public ResponseEntity<RawGrape> getRawGrapeDetails(String id) {
        if (id == null) {
            log.info("Provided Raw Grape id is null.");
            return new ResponseEntity<>(null, HttpStatus.NOT_FOUND);
        }

        // retrieve raw grape from DB
        RawGrape rawGrape = new RawGrape();
        rawGrape.setVariety("Pálava");
        rawGrape.setAmount(200);

        // handle non-existing id
        if (rawGrape == null) {
            log.info("Could not find a Raw Grape with this id .");
            return new ResponseEntity<>(null, HttpStatus.NOT_FOUND);
        }
        return new ResponseEntity<>(rawGrape, HttpStatus.OK);
    }
    @Override
    public ResponseEntity<Void> deleteRawGrape(String id) {
        if (id == null) {
            log.info("Provided Raw Grape id is null.");
            return new ResponseEntity<>(null, HttpStatus.NOT_FOUND);
        }
        // Remove from DB
        boolean rawGrapeDeleted = true;
        // Handle deletion error
        if (!rawGrapeDeleted) {
            log.info("Deletion of this Raw Grape was not successful.");
            return new ResponseEntity<>(null, HttpStatus.UNPROCESSABLE_ENTITY);
        }
        return new ResponseEntity<>(HttpStatus.OK);
    }

    @Override
    public ResponseEntity<Void> editRawGrape(String id, EditRawGrape editRawGrape) {
        if (id == null) {
            log.info("Provided Raw Grape id is null.");
            return new ResponseEntity<>(null, HttpStatus.NOT_FOUND);
        }
        // Edit in DB
        boolean rawGrapeEdited = true;
        if (!rawGrapeEdited) {
            log.info("Could not edit Raw Grape with this id ");
            return new ResponseEntity<>(null, HttpStatus.UNPROCESSABLE_ENTITY);
        }
        return new ResponseEntity<>(HttpStatus.OK);
    }

    @Override
    public ResponseEntity<List<RawGrape>> getAllRawGrapes() {
        // Retrieve all RawGrapes from DB
        log.info("getAllRawGrapes");
        List<RawGrape> grapes = new ArrayList<>();
        RawGrape g1 = new RawGrape();
        g1.setVariety("Pálava");
        g1.setAmount(200);
        grapes.add(g1);
        RawGrape g2 = new RawGrape();
        g2.setVariety("Chardonay");
        g2.setAmount(1000);
        grapes.add(g2);

        return new ResponseEntity<>(grapes, HttpStatus.OK);
    }
    
}
