package cz.fi.muni;

import cz.fi.muni.inventory.api.OrderApiDelegate;
import cz.fi.muni.inventory.model.Order;
import cz.fi.muni.inventory.model.OrderItem;
import cz.fi.muni.inventory.model.OrderResponse;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;

@Component
public class OrderController implements OrderApiDelegate {
    private static final Logger log = LoggerFactory.getLogger(StockWineController.class);

    /**
     * POST /order : Buy a wine
     *
     * @param orderItem (optional)
     * @return Order accepted (status code 201)
     * or Unexpected error when creating an order. (status code 200)
     * @see OrderApi#buyWines
     */
    @Override
    public ResponseEntity<OrderResponse> buyWines(Order order) {
        if (order == null) {
            return new ResponseEntity<>(null, HttpStatus.BAD_REQUEST);
        }
        // edit in stock
        // add order to db

        return new ResponseEntity<>(new OrderResponse() {
            {
                message("Order was accepted.");
            }
        }, HttpStatus.OK);
    }

    /**
     * GET /orders : Get all orders
     * Get all wine orders
     *
     * @return OK (status code 200)
     * @see OrderApi#getAllOrders
     */
    @Override
    public ResponseEntity<List<Order>> getAllOrders() {
        // retrieve orders from db
        Order order = createPlaceholderOrder();
        List<Order> orders = new ArrayList<>();
        orders.add(order);

        return new ResponseEntity<>(orders, HttpStatus.OK);
    }

    /**
     * GET /order/{id} : Get order
     * Get order info by id
     *
     * @param id (required)
     * @return OK (status code 200)
     * @see OrderApi#getOrder
     */
    @Override
    public ResponseEntity<Order> getOrder(String id) {
        if (id == null) {
            log.info("Provided order identifier is null.");
            return new ResponseEntity<>(null, HttpStatus.NOT_FOUND);
        }
        // retrieve wine by ID from DB
        Order order = createPlaceholderOrder();

        return new ResponseEntity<>(order, HttpStatus.OK);
    }

    private Order createPlaceholderOrder() {
        Order order = new Order();
        order.setUsername("UserName");
        order.deliveryAddress("Brno");
        List<OrderItem> items = new ArrayList<>();
        OrderItem orderItem = new OrderItem();
        orderItem.setAmount(10);
        items.add(orderItem);
        orderItem.setId("someWineId");
        order.setOrderItems(items);
        return order;
    }
}
