package cz.fi.muni;

import cz.fi.muni.internalManagement.StockWineInternal;
import cz.fi.muni.inventory.api.WineApiDelegate;
import cz.fi.muni.inventory.model.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Component;

import java.util.UUID;


import java.util.ArrayList;
import java.util.List;

@Component
public class WineController implements WineApiDelegate {
    private static final Logger log = LoggerFactory.getLogger(WineController.class);


    @Override
    public ResponseEntity<NewWineResponse> createWine(NewWine newWine) {
        if (newWine == null) {
            log.info("No wine to be created provided.");
            return new ResponseEntity<>(null, HttpStatus.BAD_REQUEST);
        }
        // generate new id
        UUID newWineUUID = UUID.randomUUID();

        boolean isCreatedInStock = StockWineInternal.createStockWine(newWineUUID);
        if (!isCreatedInStock) {
            log.debug("Could not create stock record for a new wine.");
            return new ResponseEntity<>(null, HttpStatus.UNPROCESSABLE_ENTITY);
        }
        NewWineResponse newWineResponse = new NewWineResponse() {
            {
                message(newWineUUID.toString());
            }

        };
        return new ResponseEntity<>(newWineResponse, HttpStatus.CREATED);
    }

    @Override
    public ResponseEntity<Void> deleteWine(String id) {
        if (id == null) {
            log.info("Provided wine identifier is null.");
            return new ResponseEntity<>(null, HttpStatus.NOT_FOUND);
        }
        boolean isDeletedInStock = StockWineInternal.deleteStockWine(UUID.fromString(id));
        if (!isDeletedInStock) {
            log.info("Delete from stock was not successful.");
            return new ResponseEntity<>(null, HttpStatus.UNPROCESSABLE_ENTITY);
        }
        // delete from DB
        return new ResponseEntity<>(HttpStatus.OK);
    }

    @Override
    public ResponseEntity<Void> editWine(String id, EditWine editWine) {
        if (editWine == null) {
            log.debug("No parameters to edit provided.");
            return new ResponseEntity<>(null, HttpStatus.BAD_REQUEST);
        }
        // edit in db
        return new ResponseEntity<>(HttpStatus.OK);
    }

    @Override
    public ResponseEntity<List<Wine>> getAllWines() {
        // retrieve all wines from DB
        log.info("getAllWines");
        List<Wine> wines = new ArrayList<>();
        Wine wine = new Wine();
        wine.setVariety("NewWine");
        wines.add(wine);
        return new ResponseEntity<>(wines, HttpStatus.OK);
    }

    @Override
    public ResponseEntity<Wine> getWine(String id) {
        if (id == null) {
            log.info("Provided wine identifier is null.");
            return new ResponseEntity<>(null, HttpStatus.NOT_FOUND);
        }
        // retrieve wine by ID from DB
        Wine wine = new Wine();
        wine.setVariety("Chardonnay");

        return new ResponseEntity<>(wine, HttpStatus.OK);
    }


}
