package cz.fi.muni.controller;

import cz.fi.muni.mapper.BarrelMapper;
import cz.fi.muni.production.api.BarrelApiDelegate;
import cz.fi.muni.production.model.*;
import lombok.RequiredArgsConstructor;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.NativeWebRequest;

import java.time.LocalDate;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

@Component
@RequiredArgsConstructor
public class BarrelController implements BarrelApiDelegate {
    private static final Logger log = LoggerFactory.getLogger(BarrelController.class);
    private final BarrelMapper barrelMapper;

    @Override
    public ResponseEntity<BarrelResponse> addBarrel(NewBarrel newBarrel) {
        if (newBarrel == null) {
            log.info("No barrel to be created provided.");
            return new ResponseEntity<>(null, HttpStatus.BAD_REQUEST);
        }
        //check barrel is already added to database or not
        BarrelResponse response = new BarrelResponse();
        response.setMessage("Barrel is created");
        //add to database
        return new ResponseEntity<>(response, HttpStatus.CREATED);
    }

    @Override
    public ResponseEntity<DeleteBarrel> deleteBarrel(Integer id) {
        if (id == null) {
            log.info("Provided barrel identifier is null.");
            return new ResponseEntity<>(null, HttpStatus.NOT_FOUND);
        }
        // delete from DB
        return new ResponseEntity<>(HttpStatus.OK);
    }

    @Override
    public ResponseEntity<List<StorageTank>> getAllBarrels() {
        log.info("getAllBarrels");
        //retrieve barrel from db and add to list
        List<StorageTank> barrelList = new ArrayList<>();
        StorageTank barrel = new StorageTank();
        barrel.id(0);
        barrel.setVolume(3);
        barrel.setAlcoholLevel(3.5);
        barrel.setSugarLevel(1.5);
        barrel.setAcidity(2);
        barrel.fillDate(LocalDate.now());
        barrelList.add(barrel);
        return new ResponseEntity<>(barrelList, HttpStatus.OK);
    }

    @Override
    public ResponseEntity<GetBarrel> getBarrel(Integer id) {
        if (id == null) {
            log.info("Provided barrel identifier is null.");
            return new ResponseEntity<>(null, HttpStatus.NOT_FOUND);
        }
        // retrieve barrel by ID from DB
        StorageTank barrel = new StorageTank();
        barrel.id(0);
        barrel.setVolume(3);
        barrel.setAlcoholLevel(3.5);
        barrel.setSugarLevel(1.5);
        barrel.setAcidity(2);
        barrel.fillDate(LocalDate.now());
        return new ResponseEntity<>(barrelMapper.toGetBarrel(barrel), HttpStatus.OK);
    }

    @Override
    public ResponseEntity<Void> editBarrel(Integer id, EditBarrel editBarrel) {
        if (id == null || editBarrel == null) {
            log.debug("Edit was not successful. ID: " + id + " Params: " + editBarrel);
            return new ResponseEntity<>(HttpStatus.BAD_REQUEST);
        }
        // retrieve barrel for id from db and update it
        return new ResponseEntity<>(HttpStatus.OK);
    }

}
