package cz.fi.muni.controller;

import cz.fi.muni.mapper.IngredientMapper;
import cz.fi.muni.production.api.IngredientApiDelegate;
import cz.fi.muni.production.model.*;
import lombok.RequiredArgsConstructor;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;

@Component
@RequiredArgsConstructor
public class IngredientController implements IngredientApiDelegate {
    private static final Logger log = LoggerFactory.getLogger(IngredientController.class);
    private final IngredientMapper ingredientMapper;

    @Override
    public ResponseEntity<IngredientResponse> addIngredient(NewIngredient newIngredient) {
        if (newIngredient == null) {
            log.info("No ingredient to be created provided.");
            return new ResponseEntity<>(null, HttpStatus.BAD_REQUEST);
        }
        //check ingredient is already added to database or not
        IngredientResponse response = new IngredientResponse();
        response.setMessage("Ingredient is created");
        //add to database
        return new ResponseEntity<>(response, HttpStatus.CREATED);

    }

    @Override
    public ResponseEntity<DeleteIngredient> deleteIngredient(Integer id) {
        if (id == null) {
            log.info("Provided ingredient identifier is null.");
            return new ResponseEntity<>(null, HttpStatus.NOT_FOUND);
        }
        // delete from DB
        return new ResponseEntity<>(HttpStatus.OK);
    }

    @Override
    public ResponseEntity<List<Ingredient>> getAllIngredients() {
        log.info("getAllIngredients");
        //retrieve ingredient from db and add to list
        List<Ingredient> ingredients = new ArrayList<>();
        Ingredient ingredient = new Ingredient();
        ingredient.setId(0);
        ingredient.setName("sugar");
        ingredient.setAmount(0.5);
        ingredient.setPrice(18.0);
        ingredients.add(ingredient);
        return new ResponseEntity<>(ingredients, HttpStatus.OK);

    }

    @Override
    public ResponseEntity<GetIngredient> getIngredient(Integer id) {
        if (id == null) {
            log.info("Provided ingredient identifier is null.");
            return new ResponseEntity<>(null, HttpStatus.NOT_FOUND);
        }
        // retrieve ingredient by ID from DB
        Ingredient ingredient = new Ingredient();
        ingredient.setId(0);
        ingredient.setName("sugar");
        ingredient.setAmount(0.5);
        ingredient.setPrice(18.0);
        return new ResponseEntity<>(ingredientMapper.toGetIngredient(ingredient), HttpStatus.OK);

    }

    @Override
    public ResponseEntity<Void> editIngredient(Integer id, EditIngredient editIngredient) {
        if (id == null || editIngredient == null) {
            log.debug("Edit was not successful. ID: " + id + " Params: " + editIngredient);
            return new ResponseEntity<>(HttpStatus.BAD_REQUEST);
        }
        // retrieve ingredient for id from db and update it
        return new ResponseEntity<>(HttpStatus.OK);
    }

}
