package cz.fi.muni.controller;

import cz.fi.muni.mapper.RecipeMapper;
import cz.fi.muni.production.api.RecipeApiController;
import cz.fi.muni.production.api.RecipeApiDelegate;
import cz.fi.muni.production.model.*;
import lombok.RequiredArgsConstructor;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.NativeWebRequest;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

@Component
@RequiredArgsConstructor
public class RecipeController implements RecipeApiDelegate {
    private static final Logger log = LoggerFactory.getLogger(RecipeController.class);
    private final RecipeMapper recipeMapper;

    @Override
    public ResponseEntity<RecipeResponse> addRecipe(NewRecipe recipe) {
        if (recipe == null) {
            log.info("No recipe to be created provided.");
            return new ResponseEntity<>(null, HttpStatus.BAD_REQUEST);
        }
        // check recipe is already added to database or not
        RecipeResponse response = new RecipeResponse();
        response.setMessage("Recipe is created");

        //add to database
        return new ResponseEntity<>(response, HttpStatus.CREATED);
    }

    @Override
    public ResponseEntity<List<Recipe>> getAllRecipes() {
        log.info("getAllRecipes");
        //retrieve recipe from db and add to list;
        List<Recipe> recipes = new ArrayList<>();
        Recipe recipe = new Recipe();
        recipe.setId(0);
        recipe.setTechnicalProcess("Grapes picked too early may result in tart and thin-tasting wines....");
        recipe.setGrapeWineRatio(0.5);
        recipe.setAgingTime(18);
        List<Ingredient> ingredientList = new ArrayList<>();
        Ingredient ingredient = new Ingredient();
        ingredient.setName("water");
        ingredientList.add(ingredient);
        recipe.setOtherIngredients(ingredientList);
        recipes.add(recipe);
        return new ResponseEntity<>(recipes, HttpStatus.OK);
    }

    @Override
    public ResponseEntity<GetRecipe> getRecipe(Integer id) {
        if (id == null) {
            log.info("Provided recipe identifier is null.");
            return new ResponseEntity<>(null, HttpStatus.NOT_FOUND);
        }
        // retrieve recipe by ID from DB
        Recipe recipe = new Recipe();
        recipe.setId(0);
        recipe.setTechnicalProcess("Grapes picked too early may result in tart and thin-tasting wines....");
        recipe.setGrapeWineRatio(0.5);
        recipe.setAgingTime(18);
        return new ResponseEntity<>(recipeMapper.toGetRecipe(recipe), HttpStatus.OK);
    }

    @Override
    public ResponseEntity<Void> editRecipe(Integer id, EditRecipe editRecipe) {
        if (id == null || editRecipe == null) {
            log.debug("Edit was not successful. ID: " + id + " Params: " + editRecipe);
            return new ResponseEntity<>(HttpStatus.BAD_REQUEST);
        }
        // retrieve recipe for id from db and update it
        return new ResponseEntity<>(HttpStatus.OK);
    }

}
